# Generated by Django 3.2.5 on 2021-09-21 12:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payment', '0011_alter_paymentorder_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='WebhookEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('customer_id', models.CharField(max_length=200, verbose_name='Customer ID')),
                ('event_type', models.CharField(max_length=200, verbose_name='Event Type')),
                ('data_obj', models.JSONField(verbose_name='Data Object')),
                ('event_info', models.JSONField(verbose_name='Full Event Data')),
            ],
        ),
    ]



from os import name
from django.urls import path
from .views import *

urlpatterns = [
    path('checkout/<service>', CheckoutView.as_view(), name='checkout'),
    path('stripe-success/<str:session_id>', StripeSuccessView.as_view(), name='stripe-success'),
    path('paypal-success', PaypalSuccessView.as_view(), name='paypal-success'),
    path('failed', FailView.as_view(), name='fail'),
    path('stripesub/<str:priceid>', StripeSubscription.as_view(), name='stripe-sub'),
    path('webhook', StripeWebhook.as_view(), name='stripe-webhook'),
    path('stripesub-cancel-immediately', StripeSubscriptionCancel.as_view(), name='stripe-cancel'),
    path('stripesub-cancel-end', StripeSubscriptionModify.as_view(), name='stripe-mod'),
]



# Generated by Django 3.2.5 on 2021-07-29 10:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payment', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentorder',
            name='date',
            field=models.DateField(null=True, verbose_name='Transaction Date'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-09-21 10:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payment', '0008_alter_subscription_end_date'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='subid',
            field=models.CharField(max_length=200, null=True, verbose_name='Subscription ID'),
        ),
    ]



from django.db import models
from django.contrib.auth.models import User
# Create your models here.

class BasePayModel(models.Model):
    
    email = models.EmailField(verbose_name='Email', null=True)
    session_id = models.CharField(max_length=200, verbose_name='Session ID', null=True)
    payment_success = models.BooleanField(default=False, verbose_name='Payment Success', null=True)
    payment_service = models.CharField(max_length=200, verbose_name='Payment Service Name', null=True)
    additional_info = models.TextField(verbose_name='Entire Session Data', null=True)
    
    class Meta:
        abstract = True
class PaymentOrder(BasePayModel):
    name = models.ForeignKey(User,on_delete=models.DO_NOTHING, verbose_name='Name', null=True)
    date = models.DateField(verbose_name='Transaction Date', null=True)
    
    def __str__(self) -> str:
        return str(self.name)
    
class Subscription(BasePayModel):
    name = models.OneToOneField(User,on_delete=models.DO_NOTHING, verbose_name='Name', null=True)
    date = models.DateTimeField(verbose_name='Subscription Start Date', null=True)
    customer_id = models.CharField(max_length=200, verbose_name='Customer ID', null=True)
    active = models.BooleanField(default=False, verbose_name='Subscription Active')
    end_date = models.DateTimeField(verbose_name='Subscription End Date (if cancelled by User)', blank=True, null=True)
    subid = models.CharField(max_length=200, verbose_name='Subscription ID', null=True)
    
    def __str__(self) -> str:
        return str(self.name)
    
class WebhookEvent(models.Model):
    customer_id = models.CharField(max_length=200, verbose_name='Customer ID')
    event_type = models.CharField(max_length=200, verbose_name='Event Type')
    data_obj = models.JSONField(verbose_name='Data Object')
    event_info = models.JSONField(verbose_name='Full Event Data')
    
    def __str__(self) -> str:
        return str(self.customer_id)
    


# Generated by Django 3.2.5 on 2021-07-29 11:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payment', '0002_paymentorder_date'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentorder',
            name='additional_info',
            field=models.JSONField(null=True, verbose_name='Entire Session Data'),
        ),
        migrations.AlterField(
            model_name='paymentorder',
            name='email',
            field=models.EmailField(max_length=254, null=True, verbose_name='Email'),
        ),
        migrations.AlterField(
            model_name='paymentorder',
            name='name',
            field=models.CharField(max_length=200, null=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='paymentorder',
            name='payment_service',
            field=models.CharField(max_length=200, null=True, verbose_name='Payment Service Name'),
        ),
        migrations.AlterField(
            model_name='paymentorder',
            name='payment_success',
            field=models.BooleanField(default=False, null=True, verbose_name='Payment Success'),
        ),
        migrations.AlterField(
            model_name='paymentorder',
            name='session_id',
            field=models.CharField(max_length=200, null=True, verbose_name='Session ID'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-09-21 11:00

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('payment', '0010_auto_20210921_1055'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentorder',
            name='name',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to=settings.AUTH_USER_MODEL, verbose_name='Name'),
        ),
    ]



# Django Imports
from django.http.response import HttpResponse, HttpResponseRedirect
from django.shortcuts import redirect, render
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.base import TemplateView
from django.contrib.auth.mixins import LoginRequiredMixin
from django.utils.decorators import method_decorator
from django.urls import reverse
from django.views.generic import RedirectView, View

# Model Imports
from .models import PaymentOrder, Subscription, WebhookEvent
from django.contrib.auth.models import User

# Other Imports
from .payment_conf import *
from datetime import datetime, timezone

# Third-party Imports
from stripe.api_resources import payment_method
import stripe
from paypalcheckoutsdk.core import PayPalHttpClient, SandboxEnvironment, LiveEnvironment
from paypalcheckoutsdk.orders import OrdersCreateRequest, OrdersCaptureRequest

# =============================================== #
#                    Views                        #
# =============================================== #

stripe.api_key = STRIPE_API_KEY
    
if PAYPAL_SANDBOX:
    environment = SandboxEnvironment(client_id=PAYPAL_CLIENT_KEY, client_secret=PAYPAL_SECRET_KEY)
else:
    environment = LiveEnvironment(client_id=PAYPAL_CLIENT_KEY, client_secret=PAYPAL_SECRET_KEY)
        
client = PayPalHttpClient(environment)



class CheckoutView(LoginRequiredMixin, View):
    
    stripe_url_prefix = 'http://'
    paypal_url_prefix = 'http://'
    if STRIPE_PRODUCTION:
        stripe_url_prefix = 'https://'
    if not PAYPAL_SANDBOX:
        paypal_url_prefix = 'https://'
    
    def get(self, request, service):
        if STRIPE and PAYPAL:
            if service.lower() == 'stripe':
                return redirect(self.stripecheckout(request))
            elif service.lower() == 'paypal':
                return redirect(self.paypalcheckout(request))
        elif STRIPE and not PAYPAL:
            return redirect(self.stripecheckout(request))
        elif not STRIPE and PAYPAL:
            return redirect(self.paypalcheckout(request))
        else:
            raise Exception('YOU CHOSE NEITHER PAYPAL NOR STRIPE.')
    
    def stripecheckout(self, request):
        host = request.get_host()
        payment_order = PaymentOrder.objects.create(name=request.user)
        payment_order.refresh_from_db()
        paymentid = payment_order.id
        session = stripe.checkout.Session.create(
        payment_method_types=['card'],
        line_items=[{
            'price_data': {
                'currency': STRIPE_CURRENCY,
                'product_data': {
                    'name': STRIPE_PRODUCT_NAME,
                },
                'unit_amount': STRIPE_AMOUNT,
            },
            'quantity': 1,
        }],
        mode='payment',
        client_reference_id = paymentid,
        success_url='{}{}/payment/stripe-success/{{CHECKOUT_SESSION_ID}}'.format(self.stripe_url_prefix, host),
        cancel_url='{}{}{}'.format(self.stripe_url_prefix, host, reverse('fail')),
        )
        return session.url
    
    def paypalcheckout(self, request):
        host = request.get_host()
        order = OrdersCreateRequest()
        payment_order = PaymentOrder.objects.create(name=request.user)
        payment_order.refresh_from_db()
        paymentid = payment_order.id
        order.prefer('return=representation')

        order.request_body(
            {
                "intent": "CAPTURE",
                "application_context": {
                    "return_url": '{}{}{}'.format(self.paypal_url_prefix, host, reverse('paypal-success')),
                    "cancel_url": "{}{}{}".format(self.paypal_url_prefix, host, reverse('fail')),
                },
                "purchase_units": [
                    {
                        "reference_id": paymentid,
                        "amount": {
                            "currency_code": PAYPAL_CURRENCY_CODE,
                            "value": PAYPAL_AMOUNT
                        }
                    }
                ]
            }
        )
        order_response = client.execute(order)
        paypal_url = [link.href for link in order_response.result.links][1]
        request.session['order_id'] = order_response.result.id
        return paypal_url
    


class StripeSuccessView(LoginRequiredMixin, View):
    
    def get(self, request, session_id):
        session = stripe.checkout.Session.retrieve(session_id)
        if session.mode == 'subscription':
            order = Subscription.objects.get(id=session.client_reference_id)
            customer = stripe.Customer.retrieve(session.customer)
            order.session_id = session_id
            order.email = customer.email
            order.date = datetime.now()
            order.payment_success = True
            order.additional_info = session
            order.subid = session.subscription
            order.customer_id = customer.id
            order.active = True
            order.save()
        else:
            order = PaymentOrder.objects.get(id = session.client_reference_id)
            customer = stripe.Customer.retrieve(session.customer)
            order.session_id = session_id
            order.email = customer.email
            order.payment_service = 'Stripe'
            order.date = datetime.now()
            order.payment_success = True
            order.additional_info = session
            order.save()
        return render(request, SUCCESS_TEMPLATE_PATH)

    
class PaypalSuccessView(LoginRequiredMixin, View):
    
    def get(self, request):
        orderid = request.session.get('order_id')
        order = OrdersCaptureRequest(orderid)
        order_response = client.execute(order)
        
        paymentid = order_response.result['purchase_units'][0]['reference_id']
        payment_order = PaymentOrder.objects.get(id=paymentid)
        
        payment_order.email = order_response.result.payer.email_address
        payment_order.payment_service = 'PayPal'
        payment_order.payment_success = True
        payment_order.additional_info = order_response.result.__dict__
        payment_order.save()
        return render(request, SUCCESS_TEMPLATE_PATH)
    

class FailView(LoginRequiredMixin, TemplateView):
    
    def get(self, request):
        try:
            try:
                obj = Subscription.objects.get(name=request.user)
                obj.delete()
            except:
                obj = PaymentOrder.objects.get(name=request.user)
                obj.delete()
        except Exception as e:
            raise e
        return render(request, FAIL_TEMPLATE_PATH)
    

# Subscription Views
class StripeSubscription(LoginRequiredMixin, View):

    stripe_url_prefix = 'http://'
    if STRIPE_PRODUCTION:
        stripe_url_prefix = 'https://'
        
    def get(self, request, priceid):
        host = request.get_host()
        sub_order = Subscription.objects.create(name=request.user)
        sub_order.refresh_from_db()
        paymentid = sub_order.id
        session = stripe.checkout.Session.create(
            payment_method_types = ['card'],
            line_items = [
                {
                    'price': priceid,
                    'quantity': 1,
                }
            ],
            mode = 'subscription',
            client_reference_id = paymentid,
            success_url = '{}{}/payment/stripe-success/{{CHECKOUT_SESSION_ID}}'.format(self.stripe_url_prefix, host),
            cancel_url='{}{}{}'.format(self.stripe_url_prefix, host, reverse('fail')),
        )
        return redirect(session.url)

class StripeSubscriptionCancel(LoginRequiredMixin, View):
    # Cancel Immediately
    def get(self, request):
        user = request.user
        sub_obj = Subscription.objects.get(name=user)
        sub_id = sub_obj.subid
        sub_obj.end_date = datetime.now()
        sub_obj.active = False
        sub_obj.save()
        stripe.Subscription.delete(sub_id)
        return render(request, CANCEL_TEMPLATE_PATH)

class StripeSubscriptionModify(LoginRequiredMixin, View):
    # Cancel at the end of Period
    def get(self, request):
        user = request.user
        sub_obj = Subscription.objects.get(name=user)
        sub_id = sub_obj.subid
        subinfo = stripe.Subscription.retrieve(sub_id)
        end_date = subinfo['current_period_end']
        e_date = datetime.fromtimestamp(end_date / 1.0, tz=timezone.utc)
        sub_obj.end_date = e_date
        sub_obj.save()
        stripe.Subscription.modify(sub_id, cancel_at_period_end=True)
        return render(request, DISCONTINUE_TEMPLATE_PATH)


class StripeWebhook(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)
    
    @method_decorator(csrf_exempt, name='dispatch')
    def post(self, request):
        event = None
        payload = request.body
        sig_header = request.headers['STRIPE_SIGNATURE']
        endpoint_secret = STRIPE_WEBHOOK_SECRET
        try:
            event = stripe.Webhook.construct_event(
                payload, sig_header, endpoint_secret
            )
        except ValueError as e: 
            raise e
        except stripe.error.SignatureVerificationError as e:
            raise e
        
        eventlist = [
            'invoice.finalized',
            'invoice.payment_succeeded',
            'invoice.payment_action_required',
            'invoice.payment_failed',
            'invoice.updated',
            'invoice.paid',
            'payment_intent.succeeded',
            'payment_intent.failed',
            'payment_intent.canceled',
            'payment_intent.created',
            'customer.subscription.created',
            'customer.subscription.deleted',
            'customer.subscription.updated',
            'customer.source.expiring',
            'charge.succeeded',
            'charge.failed',
            
        ]
        if event['type'] in eventlist:
            WebhookEvent.objects.create(
                customer_id=event['data']['object']['customer'],
                event_type=event['type'],
                data_obj=event['data']['object'],
                event_info=event
            )
            




# Generated by Django 3.2.5 on 2021-09-20 13:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payment', '0005_subscription'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='customer_id',
            field=models.CharField(max_length=200, null=True, verbose_name='Customer ID'),
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-29 09:41

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentOrder',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='Name')),
                ('email', models.EmailField(max_length=254, verbose_name='Email')),
                ('session_id', models.CharField(max_length=200, verbose_name='Session ID')),
                ('payment_success', models.BooleanField(default=False, verbose_name='Payment Success')),
                ('payment_service', models.CharField(max_length=200, verbose_name='Payment Service Name')),
                ('additional_info', models.JSONField(verbose_name='Entire Session Data')),
            ],
        ),
    ]



# Generated by Django 3.2.5 on 2021-07-29 11:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payment', '0003_auto_20210729_1132'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentorder',
            name='additional_info',
            field=models.TextField(null=True, verbose_name='Entire Session Data'),
        ),
    ]



from django.contrib import admin
from .models import PaymentOrder, Subscription, WebhookEvent
# Register your models here.

class SubscriptionModelAdmin(admin.ModelAdmin):
    list_display = ('name', 'date', 'active')

class PaymentModelAdmin(admin.ModelAdmin):
    list_display = ('name', 'date', 'payment_success')
    
class WebhookModelAdmin(admin.ModelAdmin):
    list_display = ('customer_id', 'event_type')

admin.site.register(PaymentOrder, PaymentModelAdmin)
admin.site.register(Subscription, SubscriptionModelAdmin)
admin.site.register(WebhookEvent, WebhookModelAdmin)


"""
Stripe API key (both sandbox or live) must be entered in STRIPE_API_KEY. 

PayPal Sandbox is by default True and in live/production servers, change this to False and update the API key if required.

Change the STRIPE_AMOUNT to your desired float with 2 decimal places (i.e. for £10, enter 1000 in the amount).
"""

from django.conf import settings

STRIPE = settings.STRIPE
PAYPAL = settings.PAYPAL

STRIPE_PRODUCTION = settings.STRIPE_PRODUCTION

STRIPE_API_KEY = settings.STRIPE_API_KEY
STRIPE_AMOUNT = settings.STRIPE_AMOUNT
STRIPE_CURRENCY = settings.STRIPE_CURRENCY
STRIPE_PRODUCT_NAME = settings.STRIPE_PRODUCT_NAME

STRIPE_WEBHOOK_SECRET = settings.STRIPE_WEBHOOK_SECRET


PAYPAL_SANDBOX = settings.PAYPAL_SANDBOX
PAYPAL_CLIENT_KEY = settings.PAYPAL_CLIENT_KEY
PAYPAL_SECRET_KEY = settings.PAYPAL_SECRET_KEY
PAYPAL_AMOUNT = settings.PAYPAL_AMOUNT
PAYPAL_CURRENCY_CODE = settings.PAYPAL_CURRENCY_CODE


SUCCESS_TEMPLATE_PATH = settings.SUCCESS_TEMPLATE_PATH
FAIL_TEMPLATE_PATH = settings.FAIL_TEMPLATE_PATH
CANCEL_TEMPLATE_PATH = settings.CANCEL_TEMPLATE_PATH
DISCONTINUE_TEMPLATE_PATH = settings.DISCONTINUE_TEMPLATE_PATH


# Generated by Django 3.2.5 on 2021-09-21 09:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('payment', '0006_subscription_customer_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='active',
            field=models.BooleanField(default=False, verbose_name='Subscription Active'),
        ),
        migrations.AddField(
            model_name='subscription',
            name='end_date',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Subscription End Date (if cancelted by User)'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='date',
            field=models.DateTimeField(null=True, verbose_name='Subscription Start Date'),
        ),
    ]
